﻿using System;
using LitJson;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Reflection;
using System.Collections;
using System.Net.Sockets;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Collections.Specialized;
using System.Web.Script.Serialization;
using System.Security.Cryptography.X509Certificates;
using System.Runtime.Serialization.Formatters.Binary;
using System.Timers;

public sealed class AllowAllAssemblyVersionsDeserializationBinder : System.Runtime.Serialization.SerializationBinder
{
    public override Type BindToType(string assemblyName, string typeName)
    {
        Type typeToDeserialize = null;
        String currentAssembly = Assembly.GetExecutingAssembly().FullName;
        // In this case we are always using the current assembly 
        assemblyName = currentAssembly;
        // Get the type using the typeName and assemblyName 
        typeToDeserialize = Type.GetType(String.Format("{0}, {1}", typeName, assemblyName));
        return typeToDeserialize;
    }
}

public enum STORETYPE
{
    PLAYSTORE = 0,
    APPSTORE = 1,
    ONESTORE = 2,
    EDIT = 9,
}

public enum PACKETTYPE
{
    ADID = 0,
    IAP = 1,
    RESULT = 2,
}

public enum RESULTTYPE
{
    SUCCESS = 0,
    FAIL = 1,
}

[Serializable]
public class BASE_PACKET
{
    public STORETYPE storeType;
    public PACKETTYPE packetType;

    public bool FromByteArray(byte[] array)
    {
        try
        {
            if (array == null || array.Length < 1)
            {
                Console.WriteLine("Error packet raw data");
                return false;
            }
            var formatter = new BinaryFormatter();
            formatter.Binder = new AllowAllAssemblyVersionsDeserializationBinder();
            MemoryStream ms = new MemoryStream(array);
            object obj = formatter.Deserialize(ms);
            BASE_PACKET packet = (BASE_PACKET)obj;
            this.storeType = packet.storeType;
            this.packetType = packet.packetType;
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
            return false;
        }
        return true;
    }
}

[Serializable]
public class RESULT_PACKET : BASE_PACKET
{
    public RESULTTYPE resultType;

    public bool FromByteArray(byte[] array)
    {
        try
        {
            if (array == null || array.Length < 1)
            {
                Console.WriteLine("Error packet raw data");
                return false;
            }
            var formatter = new BinaryFormatter();
            formatter.Binder = new AllowAllAssemblyVersionsDeserializationBinder();
            MemoryStream ms = new MemoryStream(array);
            object obj = formatter.Deserialize(ms);
            RESULT_PACKET packet = (RESULT_PACKET)obj;
            this.storeType = packet.storeType;
            this.packetType = packet.packetType;
            this.resultType = packet.resultType;
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
            return false;
        }
        return true;
    }

    public byte[] ToByteArray()
    {
        var formatter = new BinaryFormatter();
        using (var stream = new MemoryStream())
        {
            formatter.Serialize(stream, this);
            return stream.ToArray();
        }
    }
}

[Serializable]
public class IAP_PACKET : BASE_PACKET
{
    public string userId; // 요청 유저 ID
    public string packageName; // 패키지명
    public string orderNumber; // 결제 번호
    public string productId; // 상품 ID
    public long purchaseTime; // 결제 시간
    public string purchaseToken; // 결제 인증 
    public bool testCode; // 테스트 여부

    // IOS example ("bklfirst", "com.test.test", "000-000-0000", "item01", "결제후 받은 JSON데이터중 Payload 데이터 4K를 넣는다", true)
    // Android example ("bklfirst", "com.test.test", "000-000-0000", "item01", "결제후 받은 JSON데이터중 purchaseToken 데이터를 넣는다", true)
    // ONE스토어는 추후 추가 예정

    public IAP_PACKET(string userid, string packagename, string ordernumber, string productid, string purchasetoken, bool testcode = false)
    {
        packetType = PACKETTYPE.IAP;
        userId = userid;
        orderNumber = ordernumber;
        packageName = packagename;
        productId = productid;
        purchaseTime = 0;
        purchaseToken = purchasetoken;
        testCode = testcode;
    }

    public IAP_PACKET()
    {
        packetType = PACKETTYPE.IAP;
        purchaseTime = 0;
        testCode = false;
    }

    public string UserId()
    {
        return userId;
    }

    public string OrderNumber()
    {
        return orderNumber;
    }

    public string PackageName()
    {
        return packageName;
    }

    public string ProductId()
    {
        return productId;
    }

    public long PurchaseTime()
    {
        return purchaseTime;
    }

    public string PurchaseToken()
    {
        return purchaseToken;
    }

    public bool TestCode()
    {
        return testCode;
    }

    public byte[] ToByteArray()
    {
        Monitor.Enter(this);
        var formatter = new BinaryFormatter();
        using (var stream = new MemoryStream())
        {
            formatter.Serialize(stream, this);
            Monitor.Exit(this);
            return stream.ToArray();
        }
    }

    public bool FromByteArray(byte[] array)
    {
        try
        {
            Monitor.Enter(this);
            if (array == null || array.Length < 1)
            {
                Console.WriteLine("Error packet raw data");
                return false;
            }
            var formatter = new BinaryFormatter();
            formatter.Binder = new AllowAllAssemblyVersionsDeserializationBinder();
            MemoryStream ms = new MemoryStream(array);
            object obj = formatter.Deserialize(ms);
            IAP_PACKET packet = (IAP_PACKET)obj;
            this.storeType = packet.storeType;
            this.packetType = packet.packetType;
            this.userId = packet.userId;
            this.productId = packet.productId;
            this.orderNumber = packet.orderNumber;
            this.packageName = packet.packageName;
            this.purchaseTime = packet.purchaseTime;
            this.purchaseToken = packet.purchaseToken;
            this.testCode = packet.testCode;
            Monitor.Exit(this);
        }
        catch (Exception e)
        {
            Monitor.Exit(this);
            Console.WriteLine(e.ToString());
            return false;
        }
        return true;
    }
}

public class IAPSERVER
{
    public static int Main(String[] args)
    {
        GoogleIAPServer.listPacket = new List<BASE_PACKET>();
        GoogleIAPServer.listSocket = new List<Socket>();
        GoogleIAPServer server = new GoogleIAPServer();        
        Thread th = new Thread(new ThreadStart(server.LogProcess));
        th.Start();
        server.StartListening();
        return 0;
        
    }
}

public enum VerifyErrorCode
{
    NotExistConfigInfo = 0,
    InvalidReceiptData = 1,
    InvalidReceiveData = 2,
    Success = 3,
    Unknown = 4,
}

public class GoogleIAPServer
{
    public static List<Socket> listSocket;
    public static List<BASE_PACKET> listPacket;

    static private string Url_Andriod_Auth = "https://www.googleapis.com/auth/androidpublisher";
    static private string Url_Andriod_Token = "https://accounts.google.com/o/oauth2/token";
    // 추후 2버전으로 변경 요망
    static private string Url_Andriod = "https://www.googleapis.com/androidpublisher/v1.1/applications/";

    static private string Url_Ios = "https://buy.itunes.apple.com/verifyReceipt";
    static private string url_Ios_Sandbox = "https://sandbox.itunes.apple.com/verifyReceipt";

    static private string Url_OneStore = "https://iap.tstore.co.kr/digitalsignconfirm.iap";
    static private string Url_OneStore_Dev = "https://iapdev.tstore.co.kr/digitalsignconfirm.iap";
    static private string ONESTORE_RECEIPT_VERIFY_FORMAT = "{{\"txid\" : \"{0}\", \"appid\" : \"{1}\", \"signdata\" : \"{2}\"}}";

    private static string Base64UrlEncode(byte[] input)
    {
        var output = Convert.ToBase64String(input);
        output = output.Split('=')[0]; // Remove any trailing '='s
        output = output.Replace('+', '-'); // 62nd char of encoding
        output = output.Replace('/', '_'); // 63rd char of encoding
        return output;
    }

    private static int[] GetExpiryAndIssueDate()
    {
        var utc0 = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
        var issueTime = DateTime.UtcNow;

        var iat = (int)issueTime.Subtract(utc0).TotalSeconds;
        var exp = (int)issueTime.AddMinutes(55).Subtract(utc0).TotalSeconds;

        return new[] { iat, exp };
    }

    public static dynamic GetAccessToken(string clientIdEMail, string keyFilePath, string szScope)
    {
        // certificate
        var certificate = new X509Certificate2(keyFilePath, "notasecret");
        // header
        var header = new { alg = "RS256", typ = "JWT" };
        // claimset
        var times = GetExpiryAndIssueDate();
        var claimset = new
        {
            iss = clientIdEMail,
            scope = szScope,
            aud = Url_Andriod_Token,
            iat = times[0],
            exp = times[1],
        };

        JavaScriptSerializer ser = new JavaScriptSerializer();
        // encoded header
        var headerSerialized = ser.Serialize(header);
        var headerBytes = Encoding.UTF8.GetBytes(headerSerialized);
        var headerEncoded = Base64UrlEncode(headerBytes);

        // encoded claimset
        var claimsetSerialized = ser.Serialize(claimset);
        var claimsetBytes = Encoding.UTF8.GetBytes(claimsetSerialized);
        var claimsetEncoded = Base64UrlEncode(claimsetBytes);

        // input
        var input = headerEncoded + "." + claimsetEncoded;
        var inputBytes = Encoding.UTF8.GetBytes(input);

        // signiture
        var rsa = certificate.PrivateKey as RSACryptoServiceProvider;
        var cspParam = new CspParameters
        {
            KeyContainerName = rsa.CspKeyContainerInfo.KeyContainerName,
            KeyNumber = rsa.CspKeyContainerInfo.KeyNumber == KeyNumber.Exchange ? 1 : 2
        };
        var aescsp = new RSACryptoServiceProvider(cspParam) { PersistKeyInCsp = false };
        var signatureBytes = aescsp.SignData(inputBytes, "SHA256");
        var signatureEncoded = Base64UrlEncode(signatureBytes);

        // jwt
        var jwt = headerEncoded + "." + claimsetEncoded + "." + signatureEncoded;

        var client = new WebClient();
        client.Encoding = Encoding.UTF8;
        var uri = Url_Andriod_Token;
        var content = new NameValueCollection();

        content["assertion"] = jwt;
        content["grant_type"] = "urn:ietf:params:oauth:grant-type:jwt-bearer";

        string response = Encoding.UTF8.GetString(client.UploadValues(uri, "POST", content));

        var result = ser.Deserialize<dynamic>(response);

        return result;
    }

    // productID 프로덕트 아이디 결제 코드
    // token 결제 토큰
    // packageName 패키지명을 기입 com.test.test
    private bool RequestVerifyFromGoogleStore(IAP_PACKET iap)
    {
        try
        {
            string productID = iap.productId;
            string token = iap.purchaseToken;
            string packageName = iap.packageName;
            string currentPath = System.IO.Directory.GetCurrentDirectory();
            var auth = GoogleIAPServer.GetAccessToken("m-noxent@nox-7787422.iam.gserviceaccount.com", @"IAP_API_NOX_ANDROID.p12", GoogleIAPServer.Url_Andriod_Auth);

            String URL = Url_Andriod + packageName + "/inapp/" + productID + "/purchases/" + token + "?access_token=" + auth["access_token"];

            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(URL);
            req.Method = "GET";
            req.Accept = "application/json";
            WebResponse res = req.GetResponse();
            StreamReader reader = new StreamReader(res.GetResponseStream(), Encoding.UTF8);
            string result = reader.ReadToEnd();
            //Console.Write(result);

            JsonData resultjson = JsonMapper.ToObject<JsonData>(result);
            string purchaseState = "";
            if(resultjson.Keys.Contains("purchaseState"))
            {
                purchaseState = resultjson["purchaseState"].ToString();
            }

            /*if (resultjson.Keys.Contains("purchaseTime"))
            {
                iap.purchaseTime = long.Parse(resultjson["purchaseTime"].ToString());
            }*/

            if (purchaseState == "0")
            {
                return true;
            } else
            {
                return false;
            }            
        }
        catch (Exception e)
        {
            Console.Write(e + "\n");
            return false;
        }
    }

    private bool RequestVerifyFromApple(bool sandbox, string productId, string token, string packageName)
    {
        string result = "";
        VerifyErrorCode code = VerifyApple(sandbox, packageName, productId, token, ref result);
        if(code == VerifyErrorCode.Success)
        {
            return true;
        } else
        {
            return false;
        }
    }

    // bool sandbox 샌드박스사용유무
    // string appid 번들ID
    // string pid 상품ID
    // string receiptData 영수증
    static private VerifyErrorCode VerifyApple(bool sandbox, string appid, string pid, string data, ref string result)
    {
        try
        {
            // Verify the receipt with Apple  
            string postString = String.Format("{{ \"receipt-data\" : \"{0}\" }}", data);
            ASCIIEncoding ascii = new ASCIIEncoding();
            byte[] postBytes = ascii.GetBytes(postString);
            HttpWebRequest request;
            request = WebRequest.Create(sandbox ? url_Ios_Sandbox : Url_Ios) as HttpWebRequest;
            request.Method = "POST";
            request.ContentType = "application/json";
            request.ContentLength = postBytes.Length;
            Stream postStream = request.GetRequestStream();
            postStream.Write(postBytes, 0, postBytes.Length);
            postStream.Close();
            HttpWebResponse response = request.GetResponse() as HttpWebResponse;
            StringBuilder sb = new StringBuilder();
            byte[] buf = new byte[8192];
            Stream resStream = response.GetResponseStream();
            string tempString = null;
            int count = 0;
            do
            {
                count = resStream.Read(buf, 0, buf.Length);
                if (count != 0)
                {
                    tempString = Encoding.ASCII.GetString(buf, 0, count);
                    sb.Append(tempString);
                }
            } while (count > 0);

            // 결과값 출력
            Console.WriteLine(sb.ToString());
            JsonData parser = JsonMapper.ToObject<JsonData>(sb.ToString());
            if (parser == null)
            {
                //Console.WriteLine("VerifyErrorCode.InvalidReceiveData 1");
                return VerifyErrorCode.InvalidReceiveData;
            } else if (parser.Keys.Contains("status") == false || parser["status"] == null || parser["status"].ToString() == "")
            {
                //Console.WriteLine("VerifyErrorCode.InvalidReceiveData 2");
                return VerifyErrorCode.InvalidReceiveData;
            }

            // 샌드박스와 리얼 검증 서버가 다른경우 21007 값이 넘어 오면 두 검증 서버 사이의 스위칭을 해야함

            if (sandbox == false && parser["status"].ToString() == "21007")
            {
                return VerifyApple(true, appid, pid, data, ref result);
            }
            else if (parser["status"].ToString() != "0")
            {
                result = parser["status"].ToString();
                //Console.WriteLine("VerifyErrorCode.InvalidReceiptData 3");
                return VerifyErrorCode.InvalidReceiptData;
            } else if (parser.Keys.Contains("receipt") == false || parser["receipt"] == null || parser["receipt"].ToString() == "")
            {
                //Console.WriteLine("VerifyErrorCode.InvalidReceiveData 4");
                return VerifyErrorCode.InvalidReceiveData;
            }

            return VerifyErrorCode.Success;
        }
        catch (Exception ex)
        {
            // We crashed and burned -- do something intelligent 
            return VerifyErrorCode.Unknown;
        }
    }

    static bool RequestVerifyFromONEStore(IAP_PACKET iap, bool test = false)
    {
        try
        {
            string postString = String.Format(ONESTORE_RECEIPT_VERIFY_FORMAT, iap.OrderNumber(), iap.PackageName(), iap.PurchaseToken());
            //string postString = String.Format(ONESTORE_RECEIPT_VERIFY_FORMAT, txid, appid, data);
            ASCIIEncoding ascii = new ASCIIEncoding();
            byte[] postBytes = ascii.GetBytes(postString);
            HttpWebRequest request;

            if (!test)
            {
                request = WebRequest.Create(Url_OneStore) as HttpWebRequest;
            }
            else
            {
                request = WebRequest.Create(Url_OneStore_Dev) as HttpWebRequest;
            }

            request.Method = "POST";
            request.ContentType = "application/json";
            request.ContentLength = postBytes.Length;
            Stream postStream = request.GetRequestStream();
            postStream.Write(postBytes, 0, postBytes.Length);
            postStream.Close();

            HttpWebResponse response = request.GetResponse() as HttpWebResponse;
            StringBuilder sb = new StringBuilder();
            byte[] buf = new byte[8192];
            Stream resStream = response.GetResponseStream();
            string tempString = null;

            int count = 0;

            do
            {
                count = resStream.Read(buf, 0, buf.Length);
                if (count != 0)
                {
                    tempString = Encoding.ASCII.GetString(buf, 0, count);
                    sb.Append(tempString);
                }
            } while (count > 0);
            //Console.WriteLine(sb.ToString());
            JsonData fd = JsonMapper.ToObject<JsonData>(sb.ToString());
            try
            {
                resStream.Close();
                response.Close();
            }
            catch
            {
            }

            string strResult = fd["status"].ToString();
            string strDetail = fd["detail"].ToString();

            // Receipt not valid
            if (strResult == null || strResult != "0" || strDetail == null || strDetail != "0000")
            {
                return false;
            }
        }
        catch (Exception ex)
        {
            return false;
        }
        return true;
    }

    // 수신 대기 상태로 변경
    public void StartListening()
    {
        int port = 11930;

        IPHostEntry ipHostInfo = Dns.Resolve(Dns.GetHostName());
        IPAddress ipAddress = ipHostInfo.AddressList[0];
        Console.WriteLine(ipAddress);
        IPEndPoint localEndPoint = new IPEndPoint(ipAddress, port);

        Socket listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
        try
        {
            listener.Bind(localEndPoint);
            listener.Listen(1000);
            Console.WriteLine("IAP Server Starting...");
            Console.WriteLine("Waiting for a connection...");

            while (true)
            {
                Socket handler = listener.Accept();
                Thread th = new Thread(new ParameterizedThreadStart(ProcessClient));
                th.Start((Object)handler);
                listSocket.Add(handler);
            }
            listener.Close();
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
        }
        Console.WriteLine("\nPress ENTER to continue...");
        Console.Read();
    }


    public void ProcessClient(Object handle)
    {
        try
        {
            Socket socket = (Socket)handle;

            BASE_PACKET basePKT = new BASE_PACKET();

            RESULT_PACKET result = new RESULT_PACKET();
            result.packetType = PACKETTYPE.RESULT;
            result.resultType = RESULTTYPE.FAIL;

            byte[] PrePacket = new byte[4];

            int bytesRec = socket.Receive(PrePacket);

            if (bytesRec <= 0)
            {
                Console.WriteLine(socket.RemoteEndPoint.ToString() + " Error Bytes == 0");
                byte[] data_size = new byte[4];
                data_size = BitConverter.GetBytes(result.ToByteArray().Length);
                socket.Send(data_size);
                socket.Send(result.ToByteArray());
                listSocket.Remove(socket);
                socket.Shutdown(SocketShutdown.Both);
                socket.Close();
            }
            else
            {
                //Console.WriteLine(bytesRec.ToString() + " Bytes Received!");
            }

            int PacketSize = BitConverter.ToInt32(PrePacket, 0);
            Console.WriteLine(PacketSize.ToString() + " Will Bytes Received!");

            byte[] MainPacket = new byte[PacketSize];

            int offset = 0;
            while (true)
            {
                //Console.WriteLine(offset.ToString() + " " + PacketSize.ToString());
                int bytes = socket.Receive(MainPacket, offset, PacketSize, SocketFlags.None);
                offset += bytes;
                PacketSize -= bytes;                

                if (PacketSize <= 0)
                {
                    break;
                }
            }

            if (basePKT.FromByteArray(MainPacket) == false)
            {
                Console.WriteLine(socket.RemoteEndPoint.ToString() + " FromByteArray Error");
                byte[] data_size = new byte[4];
                data_size = BitConverter.GetBytes(result.ToByteArray().Length);
                socket.Send(data_size);
                socket.Send(result.ToByteArray());
                listSocket.Remove(socket);
                socket.Shutdown(SocketShutdown.Both);
                socket.Close();
                return;
            }

            // 스토어 싱크
            result.storeType = basePKT.storeType;

            // 플레이 스토어 인앱 결제 검증 요청
            if (basePKT.storeType == STORETYPE.PLAYSTORE && basePKT.packetType == PACKETTYPE.IAP)
            {
                IAP_PACKET iap = new IAP_PACKET();
                iap.FromByteArray(MainPacket);
                Console.WriteLine(socket.RemoteEndPoint.ToString() + "\t" + iap.storeType.ToString() + "\t" + iap.UserId() + "\t" + iap.ProductId());
                if (true == RequestVerifyFromGoogleStore(iap))
                {
                    // 성공
                    result.resultType = RESULTTYPE.SUCCESS;
                    Console.WriteLine("PlayStore Pay Success!");
                    Monitor.Enter(this);
                    try
                    {
                        listPacket.Add(iap);
                    }
                    finally
                    {
                        Monitor.Exit(this);
                    }
                }
                else
                {
                    Console.WriteLine("PlayStore Pay Failed!");
                    //실패
                }
            }
            // 앱스토어 인앱 검증 요청
            else if (basePKT.storeType == STORETYPE.APPSTORE && basePKT.packetType == PACKETTYPE.IAP)
            {
                IAP_PACKET iap = new IAP_PACKET();
                iap.FromByteArray(MainPacket);

                Console.WriteLine(socket.RemoteEndPoint.ToString() + "\t" + iap.storeType.ToString() + "\t" + iap.UserId() + "\t" + iap.ProductId());
                if (true == RequestVerifyFromApple(iap.TestCode(), iap.ProductId(), iap.PurchaseToken(), iap.PackageName()))
                {
                    // 성공
                    result.resultType = RESULTTYPE.SUCCESS;
                    Console.WriteLine("AppStore Pay Success!");
                    Monitor.Enter(this);
                    try
                    {
                        listPacket.Add(iap);
                    }
                    finally
                    {
                        Monitor.Exit(this);
                    }
                }
                else
                {
                    Console.WriteLine("AppStore Pay Failed!");
                    //실패
                }
            }
            // 원스토어 인앱 검증 요청
            else if(basePKT.storeType == STORETYPE.ONESTORE && basePKT.packetType == PACKETTYPE.IAP)
            {
                IAP_PACKET iap = new IAP_PACKET();
                iap.FromByteArray(MainPacket);
                Console.WriteLine(socket.RemoteEndPoint.ToString() + "\t" + iap.storeType.ToString() + "\t" + iap.UserId() + "\t" + iap.ProductId());
                if (true == RequestVerifyFromONEStore(iap))
                //if (true == RequestVerifyFromONEStore(iap.PackageName(), iap.OrderNumber(), iap.PurchaseToken(), iap.TestCode()))
                {
                    // 성공
                    result.resultType = RESULTTYPE.SUCCESS;
                    Console.WriteLine("OneStore Pay Success!");
                    Monitor.Enter(this);
                    try
                    {
                        listPacket.Add(iap);
                    }
                    finally
                    {
                        Monitor.Exit(this);
                    }
                }
                else
                {
                    Console.WriteLine("OneStore Pay Failed!!");
                    //실패
                }
            }
            byte[] d_size = new byte[4];
            d_size = BitConverter.GetBytes(result.ToByteArray().Length);
            socket.Send(d_size);
            socket.Send(result.ToByteArray());
            listSocket.Remove(socket);
            socket.Shutdown(SocketShutdown.Both);
            socket.Close();
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
        }
    }

    public void LogProcess()
    {
        while (true)
        {
            if (listPacket.Count > 0)
            {
                Monitor.Enter(this);
                try
                {
                    for (int i = 0; i < listPacket.Count; i++)
                    {
                        switch (listPacket[i].packetType)
                        {
                            case (PACKETTYPE.IAP):
                                {
                                    IAP_PACKET iap = (IAP_PACKET)listPacket[i];
                                    SaveIAP(iap);
                                }
                                break;
                        }
                    }
                    listPacket.Clear();
                }
                finally
                {
                    Monitor.Exit(this);
                }
            }
            else
            {
                System.Threading.Thread.Sleep(1000);
            }
        }
    }

    public void SaveIAP(IAP_PACKET packet)
    {
        DateTime time = DateTime.Now;
        string date = time.Year.ToString("0000") + time.Month.ToString("00") + time.Day.ToString("00");

        string rootpath = System.Environment.CurrentDirectory;
        string devicepath = "\\" + packet.storeType.ToString();
        string subpath = "\\" + packet.PackageName();
        string filename = "\\" + date;
        string pathDevice = rootpath + devicepath;
        string pathPackage = pathDevice + subpath;
        string totalpath = pathPackage + filename + ".txt";
        DirectoryInfo di1 = new DirectoryInfo(pathDevice);
        if (di1.Exists == false)
        {
            di1.Create();
        }
        DirectoryInfo di2 = new DirectoryInfo(pathPackage);
        if (di2.Exists == false)
        {
            di2.Create();
        }
        FileStream fs = new FileStream(totalpath, FileMode.Append, FileAccess.Write);
        StreamWriter sw = new StreamWriter(fs, Encoding.UTF8);
        sw.WriteLine(String.Format("{0}\t{1}\t{2}\t{3}\n",  packet.orderNumber, DateTime.Now.ToString(), packet.productId, packet.userId));
        sw.Flush();
        sw.Close();
        fs.Close();
    }
}
